@@ -23,6 +23,20 @@
 * Change log:
 *
 * $Log: dvgrab.cc,v $
+* Revision 1.24  2004/07/22 02:29:06  ddennedy
+* remove bogus patch suffix on version
+*
+* Revision 1.23  2004/07/22 02:25:14  ddennedy
+* added --timesys option
+*
+* Revision 1.22  2004/07/14 03:24:20  ddennedy
+* added dv1394.h, fixed gcc 3.4 compilation and cleanup statics in ieee1394io, applied patchfor csize and cmincutsize options
+*
+*
+* Revision 1.21patch-3 2004/06/12 pmdumuid
+* add more splitting options (--csize & --csizecutmin) -- improved over patch "1.21patch-3 2004/06/11 pmdumuid"
+* Cleaned up a bit of formatting of the --help info
+*
 * Revision 1.21  2004/01/14 13:46:29  ddennedy
 * bugfix --every
 *
@@ -136,13 +150,14 @@ FileHandler *DVgrab::m_writer;
 DVgrab::DVgrab( int argc, char *argv[] ) :
 		m_port( -1 ), m_node( -1 ), m_reader_active( false ), m_autosplit( false ),
 		m_timestamp( false ), m_channel( 63 ), m_frame_count( 0 ), m_max_file_size( 1000 ),
+                m_sizesplitmode ( 0 ), m_collection_size( 0 ), m_collection_min_cut_file_size( 1 ), 
 		m_file_format( AVI_DV2_FORMAT ), m_open_dml( false ), m_frame_every( 1 ),
 		m_dv1394( NULL ), m_jpeg_quality( 75 ),
 		m_jpeg_deinterlace( false ), m_jpeg_width( -1 ), m_jpeg_height( -1 ),
 		m_jpeg_overwrite( false ), m_captureActive( false ), m_avc( NULL ), m_reader( NULL ),
 		m_dropped_frames( 0 ), m_interactive( false ), m_buffers( 100 ), m_total_frames( 0 ),
 		m_duration( "" ), m_timeDuration( NULL ), m_stdin( false ), m_noavc( false ),
-		m_guid( 0 )
+		m_guid( 0 ), m_timesys( false )
 {
 	m_frame = NULL;
 	m_writer = NULL;
@@ -244,6 +259,10 @@ void DVgrab::getargs( int argc, char *argv[] )
 		{
 			m_timestamp = true;
 		}
+		else if ( strcmp( "--timesys", argv[ i ] ) == 0 )
+		{
+			m_timesys = true;
+		}
 		else if ( ( strcmp( "--format", argv[ i ] ) == 0 ) && ( i < argc ) )
 		{
 			i++;
@@ -294,6 +313,24 @@ void DVgrab::getargs( int argc, char *argv[] )
 				exit( EXIT_FAILURE );
 			}
 		}
+		else if ( ( strcmp( "--csize", argv[ i ] ) == 0 ) && ( i < argc ) )
+		{	       
+			i++;
+			if ( sscanf( argv[ i ], "%d", &m_collection_size ) != 1 )
+                        {
+				usage();
+				exit( EXIT_FAILURE );
+			}
+		}
+		else if ( ( strcmp( "--cmincutsize", argv[ i ] ) == 0 ) && ( i < argc ) )
+		{
+			i++;
+			if ( sscanf( argv[ i ], "%d", &m_collection_min_cut_file_size ) != 1 )
+			{
+				usage();
+				exit( EXIT_FAILURE );
+			}
+		}
 		else if ( ( strcmp( "--frames", argv[ i ] ) == 0 ) && ( i < argc ) )
 		{
 			i++;
@@ -424,12 +461,13 @@ void DVgrab::getargs( int argc, char *argv[] )
 			fprintf( stderr, "  --noavc           disable use of AV/C VTR control\n" );
 			fprintf( stderr, "  --autosplit       start a new file when a new recording is detected\n" );
 			fprintf( stderr, "  --timestamp       put the date and time of recording into the file name.\n" );
+			fprintf( stderr, "  --timesys         put the system date and time into the file name.\n" );
 			fprintf( stderr, "  --format dv1      save as a 'Type 1' DV AVI file. %s\n", m_file_format == AVI_DV1_FORMAT ? "This is the default." : " " );
-			fprintf( stderr, "  --format dv2      save as a 'Type 2' DV AVI file. %s\n", m_file_format == AVI_DV2_FORMAT ? "This is the default." : " " );
+			fprintf( stderr, "           dv2      save as a 'Type 2' DV AVI file. %s\n", m_file_format == AVI_DV2_FORMAT ? "This is the default." : " " );
 			fprintf( stderr, "  --opendml         use the OpenDML extensions to write large (>1GB)\n" );
 			fprintf( stderr, "                      'Type 2' DV AVI files (default %s).\n", m_open_dml ? "on" : "off" );
 			fprintf( stderr, "  --format raw      save as a raw DV file with a .dv extension.\n" );
-			fprintf( stderr, "  --format dif      save as a raw DV file with a .dif extension.\n" );
+			fprintf( stderr, "           dif      save as a raw DV file with a .dif extension.\n" );
 #ifdef HAVE_LIBQUICKTIME
 
 			fprintf( stderr, "  --format qt       save as a QuickTime movie.\n" );
@@ -445,11 +483,15 @@ void DVgrab::getargs( int argc, char *argv[] )
 #endif
 
 			fprintf( stderr, "  --frames number   max number of frames per split, 0 = unlimited (default %d).\n", m_frame_count );
-			fprintf( stderr, "  --size number     max file size per split, 0 = unlimited (default %d MB).\n", m_max_file_size );
+			// File Sizeing Options
+			fprintf( stderr, "  --size number     max file size, 0 = unlimited (default %d MB).\n", m_max_file_size );
+			fprintf( stderr, "  --csize numb      Split file when collections of files are about to exceed *number* megabytes.\n" ); 
+			fprintf( stderr, "  --cmincutsize num Minimum file size due to a collection split. (default %d MB).\n", m_collection_min_cut_file_size ); 
+
 			fprintf( stderr, "  --every number    write every n'th frame only (default all frames).\n" );
 			fprintf( stderr, "  --duration time   total capture duration specified as a SMIL time value:\n" );
 			fprintf( stderr, "                      XXX[.Y]h, XXX[.Y]min, XXX[.Y][s], XXXms,\n" );
-			fprintf( stderr, "                      [[HH:]MM:]SS[.ms], or smpte=HH[:MM[:SS[:FF]]]\n" );
+			fprintf( stderr, "                      [[HH:]MM:]SS[.ms], or smpte=[[[HH:]MM:]SS:]FF\n" );
 			fprintf( stderr, "                      (default unlimited).\n" );
 			fprintf( stderr, "  --card number     card number (default is automatic).\n" );
 			fprintf( stderr, "  --guid hex        select one of multiple DV devices by its GUID.\n" );
@@ -539,11 +581,19 @@ void DVgrab::startCapture()
 
 		}
 		m_writer->SetTimeStamp( m_timestamp );
+		m_writer->SetTimeSys( m_timesys );
 		m_writer->SetBaseName( m_dst_file_name );
 		m_writer->SetMaxFrameCount( m_frame_count );
 		m_writer->SetAutoSplit( m_autosplit );
 		m_writer->SetEveryNthFrame( m_frame_every );
 		m_writer->SetMaxFileSize( ( off_t ) m_max_file_size * ( off_t ) ( 1024 * 1024 ) );
+		if (m_collection_size) {
+		  m_sizesplitmode = 1;
+		}
+		m_writer->SetSizeSplitMode( m_sizesplitmode );
+		m_writer->SetMaxColSize( ( off_t ) ( m_collection_size ) * ( off_t ) ( 1024 * 1024 ) );
+		m_writer->SetMinColSize( ( off_t ) ( m_collection_size - m_collection_min_cut_file_size ) * ( off_t ) ( 1024 * 1024 ) );
+		
 	}
 	if ( m_avc )
 		m_avc->Play( m_node );
@@ -702,6 +752,7 @@ void DVgrab::captureThreadRun()
 							bool discontinuity = prevTimeCode.sec != -1 && ( time_diff > 1 || ( time_diff < 0 && time_diff > -59 ) );
 							if ( m_writer->FileIsOpen() && m_autosplit && ( m_frame->IsNewRecording() || discontinuity ) )
 							{
+								m_writer->CollectionCounterUpdate();
 								m_writer->Close();
 								m_frame->GetRecordingDate( recDate );
 								sendEvent( "\"%s\": %8.2f MB %d frames timecode %2.2d:%2.2d:%2.2d.%2.2d date %4.4d.%2.2d.%2.2d %2.2d:%2.2d:%2.2d",
