@@ -71,29 +71,6 @@ using std::endl;
 #include "frame.h"
 #include "error.h"
 
-/* static member variables.
- 
-   I tried non-static member variables first but they did not work
-   when used by two threads. I have no idea why though.
- 
-   CY: Because are threads are running on static methods :-/ - to
-   remove it we need to pass the object pointer into the callback
-   method (somehow).
-*/
-
-int IEEE1394Reader::droppedFrames;
-pthread_mutex_t IEEE1394Reader::mutex;
-pthread_mutex_t IEEE1394Reader::condition_mutex;
-pthread_cond_t IEEE1394Reader::condition;
-Frame *IEEE1394Reader::currentFrame;
-deque < Frame* > IEEE1394Reader::inFrames;
-deque < Frame* > IEEE1394Reader::outFrames;
-raw1394handle_t raw1394Reader::handle;
-unsigned char *dv1394Reader::m_dv1394_map;
-int dv1394Reader::m_dv1394_fd;
-raw1394handle_t AVC::avc_handle;
-pthread_mutex_t AVC::avc_mutex;
-
 /** Initializes the IEEE1394Reader object.
  
     The object is initialized with port and channel number. These
@@ -311,7 +288,7 @@ bool raw1394Reader::StartThread()
 	currentFrame = NULL;
 	if ( Open() && StartReceive() )
 	{
-		pthread_create( &thread, NULL, Thread, this );
+		pthread_create( &thread, NULL, ThreadProxy, this );
 		pthread_mutex_unlock( &mutex );
 		return true;
 	}
@@ -343,10 +320,10 @@ void raw1394Reader::StopThread()
 	pthread_mutex_lock( &mutex );
 	pthread_cancel( thread );
 	pthread_join( thread, NULL );
+	pthread_mutex_unlock( &mutex );
 	StopReceive();
 	Close();
 	Flush();
-	pthread_mutex_unlock( &mutex );
 	TriggerAction( );
 }
 
@@ -380,7 +357,8 @@ bool raw1394Reader::Open()
 		fail_neg( numcards = raw1394_get_port_info( handle, pinf, 16 ) );
 		fail_neg( raw1394_set_port( handle, port ) );
 
-		oldhandler = raw1394_set_iso_handler( handle, channel, Handler );
+		oldhandler = raw1394_set_iso_handler( handle, channel, HandlerProxy );
+		raw1394_set_userdata( handle, this );
 
 		success = true;
 
@@ -466,8 +444,13 @@ void raw1394Reader::StopReceive()
  
 */
 
-int raw1394Reader::Handler( raw1394handle_t handle, int channel, size_t length,
-                            quadlet_t *data )
+int raw1394Reader::HandlerProxy( raw1394handle_t handle, int channel, size_t length, quadlet_t *data )
+{
+	raw1394Reader *self = static_cast< raw1394Reader* >( raw1394_get_userdata( handle ) );
+	return self->Handler( channel, length, data );
+}
+
+int raw1394Reader::Handler( int channel, size_t length, quadlet_t *data )
 {
 	/* skip empty packets */
 
@@ -549,13 +532,17 @@ int raw1394Reader::Handler( raw1394handle_t handle, int channel, size_t length,
     but only in the pthread_testcancel() function.
  
 */
-void* raw1394Reader::Thread( void* arg )
+void* raw1394Reader::ThreadProxy( void* arg )
 {
-	raw1394Reader * theReader = ( raw1394Reader* ) arg;
+	raw1394Reader* self = static_cast< raw1394Reader* >( arg );
+	return self->Thread();
+}
 
+void* raw1394Reader::Thread()
+{
 	while ( true )
 	{
-		raw1394_loop_iterate( theReader->handle );
+		raw1394_loop_iterate( handle );
 		pthread_testcancel();
 	}
 	return NULL;
@@ -598,7 +585,7 @@ bool dv1394Reader::StartThread()
 	currentFrame = NULL;
 	if ( Open() && StartReceive() )
 	{
-		pthread_create( &thread, NULL, Thread, this );
+		pthread_create( &thread, NULL, ThreadProxy, this );
 		pthread_mutex_unlock( &mutex );
 		return true;
 	}
@@ -630,10 +617,10 @@ void dv1394Reader::StopThread()
 	pthread_mutex_lock( &mutex );
 	pthread_cancel( thread );
 	pthread_join( thread, NULL );
+	pthread_mutex_unlock( &mutex );
 	StopReceive();
 	Close();
 	Flush();
-	pthread_mutex_unlock( &mutex );
 }
 
 
@@ -792,13 +779,17 @@ bool dv1394Reader::Handler( int handle )
     but only in the pthread_testcancel() function.
  
 */
-void* dv1394Reader::Thread( void* arg )
+void* dv1394Reader::ThreadProxy( void* arg )
 {
-	dv1394Reader * theReader = ( dv1394Reader* ) arg;
+	dv1394Reader* self = static_cast< dv1394Reader* >( arg );
+	return self->Thread();
+}
 
+void* dv1394Reader::Thread()
+{
 	while ( true )
 	{
-		if ( ! Handler( theReader->m_dv1394_fd ) )
+		if ( ! Handler( m_dv1394_fd ) )
 			break;
 		pthread_testcancel();
 	}
@@ -1189,7 +1180,7 @@ bool pipeReader::StartThread()
 {
 	pthread_mutex_lock( &mutex );
 	currentFrame = NULL;
-	pthread_create( &thread, NULL, Thread, this );
+	pthread_create( &thread, NULL, ThreadProxy, this );
 	pthread_mutex_unlock( &mutex );
 	return true;
 }
@@ -1207,8 +1198,8 @@ void pipeReader::StopThread()
 	pthread_mutex_lock( &mutex );
 	pthread_cancel( thread );
 	pthread_join( thread, NULL );
-	Flush();
 	pthread_mutex_unlock( &mutex );
+	Flush();
 }
 
 
@@ -1260,10 +1251,14 @@ bool pipeReader::Handler()
     but only in the pthread_testcancel() function.
  
 */
-void* pipeReader::Thread( void* arg )
+void* pipeReader::ThreadProxy( void* arg )
 {
-	pipeReader * theReader = ( pipeReader* ) arg;
+	pipeReader* self = static_cast< pipeReader* >( arg );
+	return self->Thread();
+}
 
+void* pipeReader::Thread()
+{
 	while ( true )
 	{
 		if ( ! Handler() )
@@ -1278,5 +1273,3 @@ void* pipeReader::Thread( void* arg )
 	pthread_mutex_unlock( &mutex );
 	return NULL;
 }
-
-
